from typing import List

from endorser import Schema


class CustomSchema(Schema):
    str_prop: str


class ParentSchema(Schema):
    str_prop: str
    int_prop: int
    list_prop: list
    dict_prop: dict = None
    custom_obj: CustomSchema
    typed_list_prop: List[str]
    typed_list_prop_with_custom_obj: List[CustomSchema]
    prop_with_default_value: str = 'def'

    def validate_str_prop(self, value):
        return value


class InvalidSchema(Schema):
    invalid_prop = None
    str_prop: str



import functools
import uuid

from endorser.error import construct_error, ErrorNames


def not_none(validation_field):
    """
    Checks whether a value is None
    """

    @functools.wraps(validation_field)
    def validator(self, value):
        prop_name = _get_property_name_from(validation_field)
        if value is None:
            self.instance_errors.append(
                construct_error(prop_name,
                                'None value',
                                self.__class__.__name__,
                                name=ErrorNames.NONE_VALUE.value))
        return validation_field(self, value)

    return validator


def not_empty(validation_field):
    """
    Checks whether a value is empty (which is either None or an empty value).
    """

    @functools.wraps(validation_field)
    def validator(self, value):
        prop_name = _get_property_name_from(validation_field)
        if not value and value is not 0:
            self.instance_errors.append(
                construct_error(prop_name,
                                'empty value',
                                self.__class__.__name__,
                                name=ErrorNames.EMPTY_VALUE.value))
        return validation_field(self, value)

    return validator


def min_size(size: int):
    """Checks whether the value has the minimum size (<)."""

    def decorator(validator_function):
        def validator(self, value):
            prop_name = _get_property_name_from(validator_function)
            if not value or len(value) < size:
                self.instance_errors.append(
                    construct_error(prop_name,
                                    'minimum size %d not reached' % size,
                                    self.__class__.__name__,
                                    name=ErrorNames.MIN_SIZE_NOT_REACHED
                                    .value))
            return validator_function(self, value)

        return validator

    return decorator


def max_size(size: int):
    """Checks whether the value has the maximum size (>)."""

    def decorator(validator_function):
        def validator(self, value):
            prop_name = _get_property_name_from(validator_function)
            if not value or len(value) > size:
                self.instance_errors.append(
                    construct_error(prop_name,
                                    'maximum size %d exceeded' % size,
                                    self.__class__.__name__,
                                    name=ErrorNames.MAX_SIZE_EXCEEDED.value))
            return validator_function(self, value)

        return validator

    return decorator


def valid_uuid(validation_field):
    """
    Checks whether the value is a valid UUID.
    """

    @functools.wraps(validation_field)
    def validator(self, value):
        prop_name = _get_property_name_from(validation_field)
        try:
            uuid.UUID(value)
        except (ValueError, AttributeError, TypeError):
            self.instance_errors.append(
                construct_error(prop_name,
                                '{} is not a valid uuid'.format(value),
                                self.__class__.__name__,
                                name=ErrorNames.INVALID_UUID.value))
        return validation_field(self, value)

    return validator


def _get_property_name_from(fn):
    return fn.__name__.split('_', 1)[1]



import platform
import typing

from endorser.error import construct_error, ErrorNames

VERSION = platform.python_version().split(".")
PY37 = VERSION[0] is '3' and VERSION[1] is '7'
GENERIC_TYPE = typing._GenericAlias if PY37 else typing.GenericMeta


class Schema:

    def __new__(cls, **kwargs):
        """
        Collects mandatory fields into a list and binds it to the class.
        Only has to run once for every class, hence the `_processed` property.
        """
        if not hasattr(cls, '_processed'):
            mandatory_fields = []
            for k in cls.__annotations__.keys():

                # collect mandatory fields
                if not hasattr(cls, k):
                    mandatory_fields.append(k)

                # assign empty class variables from annotations
                # necessary to check for unknown attributes later on
                if not hasattr(cls, k):
                    setattr(cls, k, None)

            cls._mandatory_fields = mandatory_fields
            cls._processed = True

        return super(Schema, cls).__new__(cls)

    def __init__(self, _allow_unknown=False, **kwargs):
        """
        Initializes the `Schema` object, running provided validations.

        :param _allow_unknown: whether to allow unknown properties on the
            object
        """
        mandatory_fields = self._mandatory_fields.copy()
        self._instance_errors = []
        self._doc_errors = []

        class_items = self.__class__.__dict__
        # set attributes
        for k, v in kwargs.items():
            if not _allow_unknown:
                self._check_unknown_attributes(k)

            # remove provided attributes from the mandatory list
            # this is necessary to accept provided `None` values as well
            if k in mandatory_fields and v is not None:
                mandatory_fields.remove(k)

            # run validations
            validation_field = 'validate_%s' % k
            if validation_field in class_items:
                v = class_items[validation_field](self, v)

            # check if type matches the annotated type
            self._validate_type(k, v)

            setattr(self, k, v)

        # check if all mandatory fields have been set
        self._check_mandatory_fields(mandatory_fields)

    def _check_unknown_attributes(self, attr_name):
        """
        Checks for unknown attributes.

        :param attr_name: the attribute's name
        """
        if not hasattr(self, attr_name):
            self._instance_errors.append(
                construct_error(attr_name, "unknown attribute",
                                self.__class__.__name__,
                                name=ErrorNames.UNKNOWN_ATTRIBUTE.value))

    def _validate_type(self, attr_name, attr_val):
        """
        Validates the type of the property based on the annotation.

        :param attr_name: the name of the attribute
        :param attr_val: the value of the attribute
        """
        type_ = type(attr_val)
        try:
            annotated_type = self.__class__.__annotations__[attr_name]
        except KeyError:
            # KeyError means unknown attribute. Can only occur when
            # `_allow_unknown is True`
            return
        if attr_val is None:
            # if the value is None, it's type will be NoneType
            # we check for mandatory value assignment later
            pass
        elif type(annotated_type) is GENERIC_TYPE and \
                ((PY37 and annotated_type._name is 'List') or
                 (not PY37 and issubclass(annotated_type, list))):
            # check typing.List
            try:
                list_element_type = annotated_type.__args__[0]
                for i, elem in enumerate(attr_val):
                    if not type(elem) is list_element_type:
                        self._instance_errors.append(
                            construct_error(
                                attr_name, "wrong type in index %s. expected: "
                                           "'%s', provided: '%s'" %
                                           (i, list_element_type, type(elem)),
                                self.__class__.__name__,
                                name=ErrorNames.WRONG_TYPE.value)
                        )
                        break
            except (IndexError, TypeError):
                # if we get either of these exceptions it means that the
                # annotation is a generic List with no type hints for it's
                # content
                pass

        elif not type_ == annotated_type:
            self._instance_errors.append(
                construct_error(attr_name,
                                "wrong type. expected: '%s', provided: '%s'"
                                % (annotated_type.__name__, type_.__name__),
                                self.__class__.__name__,
                                name=ErrorNames.WRONG_TYPE.value))

    def _check_mandatory_fields(self, mandatory_fields):
        for mandatory in mandatory_fields:
            if getattr(self, mandatory) is None:
                self._instance_errors.append(
                    construct_error(mandatory,
                                    "mandatory field not set",
                                    self.__class__.__name__,
                                    name=ErrorNames.MANDATORY_FIELD_NOT_SET
                                    .value))

    @property
    def instance_errors(self):
        """
        :return: the validation errors on this object
        """
        return self._instance_errors

    @property
    def doc_errors(self):
        """
        Traverses every object in this instance (include self) and returns all
        validation errors.

        :return: validation errors for every object in this object including
            self
        """
        if self._doc_errors:
            return self._doc_errors

        errors = self._instance_errors
        for prop, val in vars(self).items():
            if issubclass(type(val), Schema):
                errors = errors + val.doc_errors
        self._doc_errors = errors
        return self._doc_errors

    def __repr__(self):
        class_dict = self.__class__.__dict__.copy()
        class_vars = {}
        for k, v in class_dict.items():
            if k[:1] != '_' and not callable(v):
                class_vars[k] = v
        class_vars.update(self.__dict__)
        return str(class_vars)



import unittest

from endorser import validator


class ValidatorTest(unittest.TestCase):
    FIELD_NAME = 'field_mock'

    def setUp(self):
        self.instance_errors = []

    def validation_field_mock(self, fun, val):
        return val

    def validation_field(self, fun, val):
        return val

    def test_not_none_validator(self):
        validator.not_none(self.validation_field_mock)(self, 'not_null')
        self.assertEqual([], self.instance_errors)

    def test_any_validator_with_validation_function_with_one_part(self):
        validator.not_none(self.validation_field)(self, None)
        self.assertEqual(1, len(self.instance_errors))
        self.assertEqual('ValidatorTest',
                         self.instance_errors[0].get('class'))
        self.assertEqual("field",
                         self.instance_errors[0].get("field"))

    def test_not_none_validator_with_null_val(self):
        validator.not_none(self.validation_field_mock)(self, None)
        self.assertEqual(1, len(self.instance_errors))
        self.assertEqual('ValidatorTest',
                         self.instance_errors[0].get('class'))
        self.assertEqual(self.FIELD_NAME,
                         self.instance_errors[0].get("field"))

    def test_not_empty_validator_with_empty_values(self):
        validator.not_empty(self.validation_field_mock)(self, '')
        validator.not_empty(self.validation_field_mock)(self, {})
        validator.not_empty(self.validation_field_mock)(self, [])
        validator.not_empty(self.validation_field_mock)(self, ())
        # check if 0 breaks it, it shouldn't
        validator.not_empty(self.validation_field_mock)(self, 0)
        self.assertEqual(4, len(self.instance_errors))
        self.assertEqual('ValidatorTest',
                         self.instance_errors[0].get('class'))
        self.assertEqual(self.FIELD_NAME,
                         self.instance_errors[0].get('field'))

    def test_not_empty_validator_with_not_empty_values(self):
        validator.not_empty(self.validation_field_mock)(self, 's')
        validator.not_empty(self.validation_field_mock)(self, {1: '1'})
        validator.not_empty(self.validation_field_mock)(self, [1])
        validator.not_empty(self.validation_field_mock)(self, (1, 2))
        self.assertEqual([], self.instance_errors)

    def test_min_size_validator(self):
        validator.min_size(5)(self.validation_field_mock)(self, '12345')
        validator.min_size(5)(self.validation_field_mock)(self, [1, 2, 3, 4, 5])
        validator.min_size(5)(self.validation_field_mock)(self, {1: 1, 2: 2,
                                                                 3: 3, 4: 4,
                                                                 5: 5})
        validator.min_size(5)(self.validation_field_mock)(self, (1, 2, 3, 4, 5))
        validator.min_size(-10)(self.validation_field_mock)(self, 'any')
        self.assertEqual([], self.instance_errors)

    def test_min_size_validator_with_invalid_value(self):
        validator.min_size(5)(self.validation_field_mock)(self, None)
        validator.min_size(5)(self.validation_field_mock)(self, '1234')
        validator.min_size(5)(self.validation_field_mock)(self, [1, 2, 3, 4])
        validator.min_size(5)(self.validation_field_mock)(self, {1: 1, 2: 2,
                                                                 3: 3, 4: 4})
        validator.min_size(5)(self.validation_field_mock)(self, (1, 2, 3, 4))
        self.assertEqual(5, len(self.instance_errors))
        self.assertEqual('ValidatorTest',
                         self.instance_errors[0].get('class'))
        self.assertEqual(self.FIELD_NAME,
                         self.instance_errors[0].get('field'))

    def test_max_size_validator(self):
        validator.max_size(3)(self.validation_field_mock)(self, '123')
        validator.max_size(3)(self.validation_field_mock)(self, [1, 2, 3])
        validator.max_size(3)(self.validation_field_mock)(self, {1: 1, 2: 2, 3: 3})
        validator.max_size(3)(self.validation_field_mock)(self, (1, 2, 3))
        self.assertEqual([], self.instance_errors)

    def test_max_size_validator_with_invalid_value(self):
        validator.max_size(3)(self.validation_field_mock)(self, None)
        validator.max_size(3)(self.validation_field_mock)(self, '1234')
        validator.max_size(3)(self.validation_field_mock)(self, [1, 2, 3, 4])
        validator.max_size(3)(self.validation_field_mock)(self, {1: 1, 2: 2,
                                                                 3: 3, 4: 4})
        validator.max_size(3)(self.validation_field_mock)(self, (1, 2, 3, 4))
        self.assertEqual(5, len(self.instance_errors))
        self.assertEqual('ValidatorTest',
                         self.instance_errors[0].get('class'))
        self.assertEqual(self.FIELD_NAME,
                         self.instance_errors[0].get('field'))

    def test_validate_uuid(self):
        validator.valid_uuid(self.validation_field_mock)(
            self, 'b0f07866-33bb-496c-98a8-49c040c1c18e')
        self.assertEqual([], self.instance_errors)

    def test_validate_uuid_with_invalid_value(self):
        validator.valid_uuid(self.validation_field_mock)(
            self, None)
        validator.valid_uuid(self.validation_field_mock)(
            self, 'invalid-uuid')
        validator.valid_uuid(self.validation_field_mock)(
            self, 123)
        validator.valid_uuid(self.validation_field_mock)(
            self, [1, 2, 3])
        validator.valid_uuid(self.validation_field_mock)(
            self, (1, 2))
        validator.valid_uuid(self.validation_field_mock)(
            self, object())
        self.assertEqual(6, len(self.instance_errors))



from typing import get_type_hints, TypeVar, Type, Union, List

from endorser.schema import Schema

S = TypeVar('S', dict, list)
T = TypeVar('T', bound=Schema)


class ConversionError(Exception):
    """Exception to raise when conversion fails."""

    def __init__(self, errors: list):
        self.errors = errors


class DocumentConverter:
    """
    Converter class to convert documents to typed objects.
    """

    def convert(self, document: S, doc_type: Union[Type[T], Type[List[T]]],
                allow_unknown=False) -> Union[T, List[T]]:
        """
        Converts an S from type list/dict to Type[T]/Type[List[T]].

        :param document: the object to convert
        :param doc_type: the class to convert to
        :param allow_unknown: whether to allow unknown values to be present
        :return: a populated class with type T
        """
        if not document:
            raise ValueError('empty document provided')

        if type(document) is dict:
            data = _transform_dict(document, doc_type, allow_unknown)
            if data.doc_errors:
                raise ConversionError(data.doc_errors)
        elif type(document) is list:
            data = _transform_list(document, doc_type, allow_unknown)
            errors = []
            for obj in data:
                if obj.doc_errors:
                    errors = errors + obj.doc_errors
            if errors:
                raise ConversionError(errors)
        else:
            raise TypeError('%s type cannot be converted, it has to be either'
                            'a list or a dict' % str(type(document)))
        return data


def _transform_dict(document: dict, doc_type: Type[T],
                    allow_unknown: bool) -> T:
    """
    Transforms the document from dict to type T.

    :param document: the data to transform
    :param doc_type: the class to transform to
    :return: the transformed object
    """
    hints = get_type_hints(doc_type)
    for k, v in document.items():
        if k not in hints:
            raise ValueError('%s is not type hinted' % k)

        if type(v) is dict:
            document[k] = _transform_dict(v, hints[k], allow_unknown)
        elif type(v) is list:
            try:
                document[k] = _transform_list(
                    v, hints[k], allow_unknown)
            except (IndexError, TypeError, AttributeError):
                # if we get either of these exceptions it means that the
                # annotation is a generic list with no type hints for it's
                # content, so we'll leave the content as it is.
                pass
    return doc_type(_allow_unknown=allow_unknown, **document)


def _transform_list(document: list, doc_type: Type[T],
                    allow_unknown: bool) -> List[T]:
    """
    Transforms the document from list to type T.

    :param document: the data to transform
    :param doc_type: the class to transform to
    :return: the transformed object
    """
    try:
        type_ = doc_type.__args__[0]
    except TypeError:
        raise TypeError('generic List type cannot be used as document type, '
                        'provide a type for the content of the list as well')
    return [_transform_dict(obj, type_, allow_unknown) for obj in
            document]



import unittest

from test.data import CustomSchema, ParentSchema


class TestSchema(unittest.TestCase):
    list_prop = [5, 6, 7]
    dict_prop = {"some": "dict"}
    int_prop = 123
    str_prop_1 = 'string_1'
    str_prop_2 = 'string_2'
    str_prop_3 = 'typed'
    str_prop_4 = 'list'
    test_schema_1 = CustomSchema(str_prop=str_prop_2)
    test_schema_2 = CustomSchema(str_prop='string_3')

    def setUp(self):
        self.PROPERTIES = {
            'str_prop': self.str_prop_1,
            'int_prop': self.int_prop,
            'list_prop': self.list_prop,
            'dict_prop': self.dict_prop,
            'custom_obj': self.test_schema_1,
            'typed_list_prop': [self.str_prop_3, self.str_prop_4],
            'typed_list_prop_with_custom_obj': [self.test_schema_2]
        }

    def test_correct_schema_creation(self):
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(schema.str_prop, self.str_prop_1)
        self.assertEqual(schema.int_prop, self.int_prop)
        self.assertEqual(schema.list_prop, self.list_prop)
        self.assertEqual(schema.dict_prop, self.dict_prop)
        self.assertEqual(schema.custom_obj, self.test_schema_1)
        self.assertCountEqual(schema.typed_list_prop, [self.str_prop_3,
                                                       self.str_prop_4])
        self.assertEqual(schema.typed_list_prop_with_custom_obj,
                         [self.test_schema_2])
        self.assertEqual(schema.prop_with_default_value, 'def')

    def test_updated_default_value(self):
        other_val = 'some_other_value'
        self.PROPERTIES['prop_with_default_value'] = other_val
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(schema.prop_with_default_value, other_val)

    def test_incorrect_type(self):
        invalid_prop = 'str_prop'
        self.PROPERTIES[invalid_prop] = 123
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(len(schema.doc_errors), 1)
        self.assertEqual(schema.doc_errors[0]['class'],
                         'ParentSchema')
        self.assertEqual(schema.doc_errors[0]['field'],
                         invalid_prop)

    def test_allow_unknown(self):
        unknown = 'unknown'
        unknown_val = 123
        self.PROPERTIES[unknown] = unknown_val
        self.PROPERTIES['_allow_unknown'] = True
        schema = ParentSchema(**self.PROPERTIES)
        self.assertEqual(schema.unknown, unknown_val)

    def test_nested_validation_errors(self):
        invalid_prop = 'str_prop'
        self.PROPERTIES[invalid_prop] = 123
        invalid_nested_prop = 'custom_obj'
        self.PROPERTIES[invalid_nested_prop] = CustomSchema(str_prop=456)
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(len(schema.doc_errors), 2)
        self.assertEqual(schema.doc_errors[0]['class'],
                         'ParentSchema')
        self.assertEqual(schema.doc_errors[0]['field'],
                         invalid_prop)
        self.assertEqual(schema.doc_errors[1]['class'],
                         'CustomSchema')
        self.assertEqual(schema.doc_errors[1]['field'],
                         invalid_prop)

    def test_mandatory_argument_with_None_provided(self):
        self.PROPERTIES['str_prop'] = None
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(len(schema.doc_errors), 1)
        self.assertEqual(schema.instance_errors[0]['class'], 'ParentSchema')
        self.assertEqual(schema.instance_errors[0]['field'], 'str_prop')

    def test_mandatory_argument_with_no_arg_provided(self):
        self.PROPERTIES.pop('str_prop')
        schema = ParentSchema(**self.PROPERTIES)

        self.assertEqual(len(schema.doc_errors), 1)
        self.assertEqual(schema.str_prop, None)
        self.assertEqual(schema.instance_errors[0]['class'], 'ParentSchema')
        self.assertEqual(schema.instance_errors[0]['field'], 'str_prop')



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name='endorser',
    version='0.23',
    description='Annotation based python object validator',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    author='Krisztian Toth',
    author_email='tkrisztiana@gmail.com',
    url='https://github.com/watsta/endorser',
    keywords='object validator validate converter convert',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
)



import warnings
from enum import Enum


def construct_error(field_name: str,
                    error: str,
                    class_name: str=None,
                    name: str = None, **kwargs) -> dict:
    """
    Constructs an error from the given params.

    :param field_name: the name of the field which failed the validation
    :param error: the error message about how the validation failed
    :param class_name: the name of the class, optional
    :param name: a unique name of the error
    :param kwargs: any additional keyword arguments which will be added to the
        response dict
    :return: an error dict
    """
    if not name:
        warnings.warn("do not construct an error message without the name "
                      "param, later it will be mandatory",
                      category=DeprecationWarning)

    error = {'field': field_name, 'error': error}

    if name:
        error['name'] = name
    if class_name:
        error['class'] = class_name
    if kwargs:
        for k, v in kwargs:
            error[k] = v
    return error


class ErrorNames(Enum):
    NONE_VALUE = "NONE_VALUE"
    EMPTY_VALUE = "EMPTY_VALUE"
    MIN_SIZE_NOT_REACHED = "MIN_SIZE_NOT_REACHED"
    MAX_SIZE_EXCEEDED = "MAX_SIZE_EXCEEDED"
    INVALID_UUID = "INVALID_UUID"
    MANDATORY_FIELD_NOT_SET = "MANDATORY_FIELD_NOT_SET"
    WRONG_TYPE = "WRONG_TYPE"
    UNKNOWN_ATTRIBUTE = "UNKNOWN_ATTRIBUTE"




import unittest
from typing import List

from endorser import ConversionError
from endorser import DocumentConverter
from test.data import ParentSchema, InvalidSchema


class ConverterTest(unittest.TestCase):
    A_STRING = "some string"
    A_STRING_2 = "another string"
    A_STRING_3 = "some string 2"
    A_STRING_4 = "another string 2"
    NOT_DEF_VAL = "not def value"

    def setUp(self):
        self.converter = DocumentConverter()
        self.VALID_DOCUMENT = {
            "str_prop": self.A_STRING,
            "int_prop": 123,
            "list_prop": ["some", 2, "values"],
            "custom_obj": {
                "str_prop": self.A_STRING_2
            },
            "typed_list_prop": ["has", "to", "be", "string"],
            "typed_list_prop_with_custom_obj": [{
                "str_prop": "values in a list 1"
            }, {
                "str_prop": "values in a list 2"
            }]
        }

        self.ANOTHER_DOCUMENT = {
            "str_prop": self.A_STRING_3,
            "int_prop": 321,
            "list_prop": [1, "value", 2],
            "custom_obj": {
                "str_prop": self.A_STRING_4
            },
            "typed_list_prop": ["string", "list"],
            "typed_list_prop_with_custom_obj": [{
                "str_prop": "values in a list 3"
            }, {
                "str_prop": "values in a list 4"
            }],
            "prop_with_default_value": self.NOT_DEF_VAL
        }

    def test_converter(self):
        result = self.converter.convert(self.VALID_DOCUMENT, ParentSchema)
        self.assertEqual(result.str_prop, self.A_STRING)
        self.assertEqual(result.custom_obj.str_prop, self.A_STRING_2)

    def test_converter_with_list(self):
        data = [self.VALID_DOCUMENT, self.ANOTHER_DOCUMENT]
        result = self.converter.convert(data, List[ParentSchema])

        self.assertEqual(2, len(result))
        self.assertEqual(result[0].str_prop, self.A_STRING)
        self.assertEqual(result[0].custom_obj.str_prop, self.A_STRING_2)
        self.assertEqual(result[0].prop_with_default_value,
                         ParentSchema.prop_with_default_value)
        self.assertEqual(result[1].str_prop, self.A_STRING_3)
        self.assertEqual(result[1].custom_obj.str_prop, self.A_STRING_4)
        self.assertEqual(result[1].prop_with_default_value, self.NOT_DEF_VAL)

    def test_converter_with_invalid_prop(self):
        prop = "int_prop"
        val = "string"
        self.VALID_DOCUMENT[prop] = val
        with self.assertRaises(ConversionError) as e:
            self.converter.convert(self.VALID_DOCUMENT, ParentSchema)
        self.assertEqual(1, len(e.exception.errors))
        self.assertEqual(e.exception.errors[0]['field'], prop)

    def test_schema_with_not_type_hinted_prop(self):
        with self.assertRaises(ValueError):
            self.converter.convert({'str_prop': 'str', 'invalid_prop': 123},
                                   InvalidSchema)
